/*
 * Copyright (C) 2012 Vadym Timofeyev
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Vadym Timofeyev - initial implementation
 */
package tvv.nano;

import java.awt.Color;

import robocode.AdvancedRobot;
import robocode.Rules;
import robocode.ScannedRobotEvent;
import robocode.util.Utils;

/**
 * Advanced robot: Polaris.
 *
 * Restrictions:
 * - robocode 1.7.3.6+
 * - maximal battle field: 1200x1200
 *
 * @author Vadym Timofeyev
 * @since 1.5
 * @version 1.1
 */
public final class Polaris extends AdvancedRobot {
    /**
     * Movement distance and direction, possible values: -160.0 and 160.0
     */
    private double direction;

    /**
     * The main method. Overridden to set up robot's basic behavior.
     */
    @Override
    public void run() {
        direction = 160.0;

        setAllColors(new Color(Constants.POLARIS_RGB));

        setAdjustGunForRobotTurn(true);

        setTurnRadarRightRadians(Double.POSITIVE_INFINITY);
    }

    /**
     * This method is called when robot sees an another robot, i.e. when the
     * robot's radar scan "hits" another robot.
     *
     * @param event The scanned-robot event set by the game.
     */
    @Override
    public void onScannedRobot(final ScannedRobotEvent event) {
        // turn radar
        setTurnRadarLeftRadians(getRadarTurnRemainingRadians());

        // turn gun
        final double bearing  = event.getBearingRadians();

        setTurnGunRightRadians(Utils.normalRelativeAngle(
            getHeadingRadians() + bearing - getGunHeadingRadians()
          + (StrictMath.random() - 0.5) * Constants.PI_DIV_4
          + StrictMath.sin(event.getHeadingRadians() - getHeadingRadians()
                         - bearing) * event.getVelocity() / 16.0
        ));

        // fire
        if (event.getDistance() < 400.0 && getGunHeat() == 0.0
         && getEnergy() > Rules.MAX_BULLET_POWER
         && Math.abs(getGunTurnRemainingRadians()) < Constants.PI_DIV_12) {
            setFire(StrictMath.max(
                StrictMath.min(400.0/event.getDistance()
                             , Rules.MAX_BULLET_POWER)
              , Rules.MIN_BULLET_POWER
            ));
        }

        // turn robot
        setTurnRightRadians(Utils.normalRelativeAngle(
            bearing + Constants.PI_DIV_2 - direction * Constants.PI_DIV_1920
        ));

        // move robot
        if (getVelocity() == 0.0) {
            setAhead(direction = -direction);
        }

        clearAllEvents();
    }
}
